# coding: utf-8
# code to plot defect diagram for a given compound from compoundname.csv
# Anuj Goyal(agoyal@nrel.gov) April 10 2010

import pandas
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
from copy import deepcopy

#############################################################################
def calculate_dfe(ps):
    """function that return defect formation energy as function of fermi-energy for each defect at given phase stability point
    
    input: ps (integer) "phase stability point"
    returns: x (float) "fermi-energy"
             y (float) "defect formation energy"
             name (string) "defect label"
    """

    #rearranging data
    name = []
    d = np.zeros([len(data_dfe), 3])

    name.append(data_dfe.values[0][0])
    
    d[0,0] = 0
    d[0,1] = data_dfe.values[0][2]
    d[0,2] = data_dfe.values[0][3]+data_dfe.values[0][4]*dmuA[ps]+data_dfe.values[0][5]*dmuB[ps]+data_dfe.values[0][6]*dmuC[ps]+data_dfe.values[0][7]*dmuD[ps]+data_dfe.values[0][8]*dmuE[ps]+data_dfe.values[0][9]*dmuF[ps]+data_dfe.values[0][10]*dmuG[ps]

    count=0

    for i in range(1, len(data_dfe)):
        if data_dfe.values[i][0] != data_dfe.values[i-1][0] or data_dfe.values[i][1] != data_dfe.values[i-1][1]:
            count=count+1
            name.append(data_dfe.values[i][0])

        d[i,0]=count
        d[i,1]=data_dfe.values[i][2]
        d[i,2]=data_dfe.values[i][3]+data_dfe.values[i][4]*dmuA[ps]+data_dfe.values[i][5]*dmuB[ps]+data_dfe.values[i][6]*dmuC[ps]+data_dfe.values[i][7]*dmuD[ps]+data_dfe.values[i][8]*dmuE[ps]+data_dfe.values[i][9]*dmuF[ps]+data_dfe.values[i][10]*dmuG[ps]

    unq = np.unique(d[:,0])
    
    #storing dH vs. EF for each unique defect
    x = np.linspace(0, eg, 1000)
    y = np.zeros([len(x), len(unq)])

    for kk in range(len(unq)):
        dummy = np.full((len(x)),1000.)
        for jj in range(len(d)):
            if d[jj,0]==unq[kk]:
                for ii in range(len(x)):
                    val = d[jj][1]*x[ii]+d[jj][2]
                    if val <= dummy[ii]: dummy[ii] = val

        y[:,kk]=dummy[:]
        
    #function returns x, y, name
    return (x,y,name)
############################################################################
if  __name__=='__main__':
    """ main code
    """

#read csv
data = pandas.read_csv('wzzno.csv')
df1 = pandas.DataFrame(data)

#find last row of DFE data
for i in range(len(data.defect)):
    if pandas.isnull(data.defect.values[i]) == False:
        end = i
        
#find last row of phasestability data
for j in range(len(data.point)):
    if pandas.isnull(data.point.values[j]) == False:
        endp = j

#read dfe at dmu[i]=0
data_dfe = pandas.concat([df1['defect'][:end+1], df1['site'][:end+1], df1['charge'][:end+1], df1['dHvbm_eV'][:end+1], df1['na'][:end+1], df1['nb'][:end+1], df1['nc'][:end+1], df1['nd'][:end+1], df1['ne'][:end+1], df1['nf'][:end+1], df1['ng'][:end+1]], axis=1)

headers_dfe = list(data_dfe)

#bandgap: contained in a metadata file
eg=data.bandgap.values[0]

#compound: contained in a metadata file
compound= data.formula[0]

#read phase stability information
data_ps = pandas.concat([df1['point'][:endp+1], df1['dmuA'][:endp+1], df1['dmuB'][:endp+1], df1['dmuC'][:endp+1], df1['dmuD'][:endp+1], df1['dmuE'][:endp+1], df1['dmuF'][:endp+1], df1['dmuG'][:endp+1]], axis=1)

headers_ps = list(data_ps)

for h in headers_ps:
    name = str(deepcopy(h))
    vars()[name] = data_ps['%s'%h].tolist()

#print("Choose point on phase stability:", [i for i in range(0,endp+1)])

#Choose phase stability point: change to get new plot
ps = 0

#annotation
chempot = 'dmuA=%s, dmuB=%s, dmuC=%s, dmuD=%s, dmuE=%s, dmuF=%s, dmuG=%s'%(dmuA[ps],dmuB[ps],dmuC[ps],dmuD[ps],dmuE[ps],dmuF[ps],dmuG[ps])

x,y,names = calculate_dfe(ps)

#plotting
plt.rc("font", size=16)
fig, ax = plt.subplots(figsize=(4.,4), dpi=100)

ax.axhline(y=0, linestyle=':', lw=0.5, color='k')

for i in range(len(y[0,:])):
    ax.plot(x,y[:,i], ls='-', lw=1.5, label=names[i])

ax.set_xlim(0, eg)

ax.set_xlabel('$E_{\mathrm{F}}$ (eV)', fontsize=18, labelpad=10)
ax.set_ylabel('$\Delta H_{\mathrm{D,q}}$ (eV)', fontsize=18, labelpad=10)
ax.set_title('%s at point %s'%(compound,ps), fontsize=16)
ax.tick_params(direction='in', which='major',length=6, pad=5, width=1,right=True, top=True)
ax.tick_params(direction='in', which='minor',length=3, pad=5, width=1,right=True, top=True)

handles, labels = ax.get_legend_handles_labels()
ax.legend(handles, labels, bbox_to_anchor=(1.0, 1.0), fontsize=12)

fig.savefig('%s.pdf'%(compound), format='pdf', bbox_inches='tight')
plt.close()

#plt.show()




